@@ -52,6 +52,7 @@
 #include "util.h"
 #include "interface.h"
 #include "selinux.h"
+#include "cgroups.h"
 
 ProcInfo::UserMap ProcInfo::users;
 ProcInfo::List ProcInfo::all;
@@ -66,7 +67,6 @@ ProcInfo* ProcInfo::find(pid_t pid)
 
 
 
-
 static void
 set_proctree_reorderable(ProcData *procdata)
 {
@@ -168,6 +168,28 @@ proctable_get_columns_order(GtkTreeView *treeview)
     return order;
 }
 
+void
+cb_proctable_column_resized(GtkWidget *widget)
+{
+    GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN(widget);
+    gint width;
+    gchar *key;
+    int id;
+    GSettings *settings;
+    gint saved_width;
+
+    settings = g_settings_get_child (ProcData::get_instance()->settings, "proctree");
+    id = gtk_tree_view_column_get_sort_column_id (column);
+    width = gtk_tree_view_column_get_width (column);
+    key = g_strdup_printf ("col-%d-width", id);
+
+    g_settings_get (settings, key, "i", &saved_width);
+    if (saved_width!=width)
+    {
+        g_settings_set_int(settings, key, width);
+    }
+    g_free (key);
+}
 
 static gboolean
 search_equal_func(GtkTreeModel *model,
@@ -223,8 +245,10 @@ proctable_new (ProcData * const procdata)
         N_("Security Context"),
         N_("Command Line"),
         N_("Memory"),
-        /* xgettext: wchan, see ps(1) or top(1) */
+        /* xgettext: combined noun, the function the process is waiting in, see wchan ps(1) */
         N_("Waiting Channel"),
+        N_("Control Group"),
+        N_("Priority"),
         NULL,
         "POINTER"
     };
@@ -254,6 +278,8 @@ proctable_new (ProcData * const procdata)
                                 G_TYPE_STRING,      /* Arguments    */
                                 G_TYPE_ULONG,       /* Memory       */
                                 G_TYPE_STRING,      /* wchan        */
+                                G_TYPE_STRING,      /* Cgroup       */
+                                G_TYPE_STRING,      /* Priority     */
                                 GDK_TYPE_PIXBUF,    /* Icon         */
                                 G_TYPE_POINTER,     /* ProcInfo     */
                                 G_TYPE_STRING       /* Sexy tooltip */
@@ -292,11 +318,12 @@ proctable_new (ProcData * const procdata)
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_min_width (column, 1);
+    g_signal_connect(G_OBJECT(column), "notify::width", G_CALLBACK(cb_proctable_column_resized), NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (proctree), column);
     gtk_tree_view_set_expander_column (GTK_TREE_VIEW (proctree), column);
 
 
-    for (i = COL_USER; i <= COL_WCHAN; i++) {
+    for (i = COL_USER; i <= COL_PRIORITY; i++) {
 
         GtkCellRenderer *cell;
         GtkTreeViewColumn *col;
@@ -307,6 +334,7 @@ proctable_new (ProcData * const procdata)
         gtk_tree_view_column_set_title(col, _(titles[i]));
         gtk_tree_view_column_set_resizable(col, TRUE);
         gtk_tree_view_column_set_sort_column_id(col, i);
+        g_signal_connect(G_OBJECT(col), "notify::width", G_CALLBACK(cb_proctable_column_resized), NULL);
         gtk_tree_view_column_set_reorderable(col, TRUE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(proctree), col);
 
@@ -350,13 +378,21 @@ proctable_new (ProcData * const procdata)
                                                         GUINT_TO_POINTER(i),
                                                         NULL);
                 break;
-
+            case COL_PRIORITY:
+                gtk_tree_view_column_set_cell_data_func(col, cell,
+                                                        &procman::priority_cell_data_func,
+                                                        GUINT_TO_POINTER(COL_NICE),
+                                                        NULL);
+                gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i,
+                                                procman::priority_compare_func,
+                                                GUINT_TO_POINTER(COL_NICE), NULL);
+                break;
             default:
                 gtk_tree_view_column_set_attributes(col, cell, "text", i, NULL);
                 break;
         }
 
-        // xaling
+        // xalign
         switch(i)
         {
             case COL_VMSIZE:
@@ -400,6 +436,14 @@ proctable_new (ProcData * const procdata)
         gtk_tree_view_column_set_visible (column, FALSE);
     }
 
+    if (!cgroups_enabled()) {
+        GtkTreeViewColumn *column;
+
+        column = my_gtk_tree_view_get_column_with_sort_column_id(GTK_TREE_VIEW(proctree), COL_CGROUP);
+        gtk_tree_view_column_set_visible(column, FALSE);
+    }
+
+
     g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (proctree))),
                       "changed",
                       G_CALLBACK (cb_row_selected), procdata);
@@ -421,6 +465,7 @@ ProcInfo::~ProcInfo()
     g_free(this->tooltip);
     g_free(this->arguments);
     g_free(this->security_context);
+    g_free(this->cgroup_name);
 }
 
 
@@ -506,7 +551,6 @@ static void get_process_memory_writable(ProcInfo *info)
     g_free(maps);
 }
 
-
 static void
 get_process_memory_info(ProcInfo *info)
 {
@@ -555,6 +599,7 @@ update_info_mutable_cols(ProcInfo *info)
     tree_store_update(model, &info->node, COL_NICE, info->nice);
     tree_store_update(model, &info->node, COL_MEM, info->mem);
     tree_store_update(model, &info->node, COL_WCHAN, info->wchan);
+    tree_store_update(model, &info->node, COL_CGROUP, info->cgroup_name);
 }
 
 
@@ -671,6 +716,9 @@ update_info (ProcData *procdata, ProcInfo *info)
     ProcInfo::cpu_times[info->pid] = info->cpu_time = proctime.rtime;
     info->nice = procuid.nice;
     info->ppid = procuid.ppid;
+
+    /* get cgroup data */
+    get_process_cgroup_info(info);
 }
 
 
@@ -715,6 +763,8 @@ ProcInfo::ProcInfo(pid_t pid)
     info->start_time = proctime.start_time;
 
     get_process_selinux_context (info);
+    info->cgroup_name = NULL;
+    get_process_cgroup_info(info);
 }
 
 
